@@ -30,48 +30,72 @@
 #ifndef MICROSIMULATION_H
 #define MICROSIMULATION_H
 
-// #pragma GCC diagnostic ignored "-Wunused-local-typedefs"
-// #pragma GCC diagnostic ignored "-Wlong-long"
-
 #include <RcppCommon.h>
-#include <boost/tuple/tuple.hpp>
-#include <boost/tuple/tuple_comparison.hpp>
-#include <boost/unordered_map.hpp>
+#include <unordered_map>
+#include <tuple>
 #include <vector>
 #include <utility>
 #include <map>
 #include <string>
 
-// http://stackoverflow.com/questions/3611951/building-an-unordered-map-with-tuples-as-keys
-namespace boost {
-  namespace tuples { // fixed cf. the source URL
-    namespace detail {
-      template <class Tuple, size_t Index = length<Tuple>::value - 1>
-	struct HashValueImpl
-	{
-	  static void apply(size_t& seed, Tuple const& tuple)
-	  {
-	    HashValueImpl<Tuple, Index-1>::apply(seed, tuple);
-	    boost::hash_combine(seed, get<Index>(tuple));
-	  }
-	};
-      template <class Tuple>
-      struct HashValueImpl<Tuple,0>
+/* https://stackoverflow.com/questions/7110301/generic-hash-for-tuples-in-unordered-map-unordered-set for a generic hash function for std::tuple
+ */
+#include <tuple>
+namespace std{
+  namespace
+  {
+    // Code from boost
+    // Reciprocal of the golden ratio helps spread entropy
+    //     and handles duplicates.
+    // See Mike Seymour in magic-numbers-in-boosthash-combine:
+    //     http://stackoverflow.com/questions/4948780
+    template <class T>
+    inline void hash_combine(std::size_t& seed, T const& v)
+    {
+      seed ^= std::hash<T>()(v) + 0x9e3779b9 + (seed<<6) + (seed>>2);
+    }
+    // Recursive template code derived from Matthieu M.
+    template <class Tuple, size_t Index = std::tuple_size<Tuple>::value - 1>
+    struct HashValueImpl
+    {
+      static void apply(size_t& seed, Tuple const& tuple)
       {
-	static void apply(size_t& seed, Tuple const& tuple)
-	{
-	  boost::hash_combine(seed, get<0>(tuple));
-	}
-      };
-    } // namespace detail
+	HashValueImpl<Tuple, Index-1>::apply(seed, tuple);
+	hash_combine(seed, std::get<Index>(tuple));
+      }
+    };
     template <class Tuple>
-    size_t hash_value(Tuple const& tuple)
+    struct HashValueImpl<Tuple,0>
+    {
+      static void apply(size_t& seed, Tuple const& tuple)
+      {
+	hash_combine(seed, std::get<0>(tuple));
+      }
+    };
+  }
+  template <typename ... TT>
+  struct hash<std::tuple<TT...>>
+  {
+    size_t
+    operator()(std::tuple<TT...> const& tt) const
     {
       size_t seed = 0;
-      detail::HashValueImpl<Tuple>::apply(seed, tuple);
+      HashValueImpl<std::tuple<TT...> >::apply(seed, tt);
       return seed;
     }
-  }
+  };
+  template <typename T1, typename T2>
+  struct hash<std::pair<T1,T2>>
+  {
+    size_t
+    operator()(std::pair<T1,T2> const& p) const
+    {
+      size_t seed = 0;
+      hash_combine(seed, p.first);
+      hash_combine(seed, p.second);
+      return seed;
+    }
+  };
 }
 
 namespace Rcpp {
@@ -82,31 +106,31 @@ namespace Rcpp {
 
   // vectors tuples: enumerated cases
   template <class T1, class T2>
-    SEXP wrap(const std::vector<boost::tuple<T1,T2> > v);
+    SEXP wrap(const std::vector<std::tuple<T1,T2> > v);
 
   template <class T1, class T2, class T3>
-    SEXP wrap(const std::vector<boost::tuple<T1,T2,T3> > v);
+    SEXP wrap(const std::vector<std::tuple<T1,T2,T3> > v);
 
   template <class T1, class T2, class T3, class T4>
-    SEXP wrap(const std::vector<boost::tuple<T1,T2,T3,T4> > v);
+    SEXP wrap(const std::vector<std::tuple<T1,T2,T3,T4> > v);
 
   template <class T1, class T2, class T3, class T4, class T5>
-    SEXP wrap(const std::vector<boost::tuple<T1,T2,T3,T4,T5> > v);
+    SEXP wrap(const std::vector<std::tuple<T1,T2,T3,T4,T5> > v);
 
   template <class T1, class T2, class T3, class T4, class T5, class T6>
-    SEXP wrap(const std::vector<boost::tuple<T1,T2,T3,T4,T5,T6> > v);
+    SEXP wrap(const std::vector<std::tuple<T1,T2,T3,T4,T5,T6> > v);
 
   template <class T1, class T2, class T3, class T4, class T5, class T6, class T7>
-    SEXP wrap(const std::vector<boost::tuple<T1,T2,T3,T4,T5,T6,T7> > v);
+    SEXP wrap(const std::vector<std::tuple<T1,T2,T3,T4,T5,T6,T7> > v);
 
   template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8>
-    SEXP wrap(const std::vector<boost::tuple<T1,T2,T3,T4,T5,T6,T7,T8> > v);
+    SEXP wrap(const std::vector<std::tuple<T1,T2,T3,T4,T5,T6,T7,T8> > v);
 
   template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9>
-    SEXP wrap(const std::vector<boost::tuple<T1,T2,T3,T4,T5,T6,T7,T8,T9> > v);
+    SEXP wrap(const std::vector<std::tuple<T1,T2,T3,T4,T5,T6,T7,T8,T9> > v);
 
   template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10>
-    SEXP wrap(const std::vector<boost::tuple<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10> > v);
+    SEXP wrap(const std::vector<std::tuple<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10> > v);
 
   // maps defined in terms of vectors
   template <class T1, class T2>
@@ -117,33 +141,33 @@ namespace Rcpp {
 		  std::string key, std::string name1, std::string name2);
 
   template <class T1a, class T1b, class T1c, class T2>
-    SEXP wrap_map(const std::map<boost::tuple<T1a,T1b,T1c>,T2> v,
+    SEXP wrap_map(const std::map<std::tuple<T1a,T1b,T1c>,T2> v,
 		  std::string key, std::string name1, std::string name2, std::string name3);
 
   template <class T1, class T2, class T3, class T4, class T5>
-  SEXP wrap_map(const std::map<std::pair<boost::tuple<T1,T2,T3>,T4>,T5> v,
+  SEXP wrap_map(const std::map<std::pair<std::tuple<T1,T2,T3>,T4>,T5> v,
 		  std::string key, std::string name1, std::string name2);
 
   // unordered_maps defined in terms of vectors
   template <class T1, class T2>
-    SEXP wrap_map(const boost::unordered_map<T1,T2> v);
+    SEXP wrap_map(const std::unordered_map<T1,T2> v);
 
   template <class T1a, class T1b, class T2>
-    SEXP wrap_map(const boost::unordered_map<std::pair<T1a,T1b>,T2> v,
+    SEXP wrap_map(const std::unordered_map<std::pair<T1a,T1b>,T2> v,
 		  std::string key, std::string name1, std::string name2);
 
   template <class T1a, class T1b, class T1c, class T2>
-    SEXP wrap_map(const boost::unordered_map<boost::tuple<T1a,T1b,T1c>,T2> v,
+    SEXP wrap_map(const std::unordered_map<std::tuple<T1a,T1b,T1c>,T2> v,
 		  std::string key, std::string name1, std::string name2, std::string name3);
 
   template <class T1, class T2, class T3, class T4, class T5>
-    SEXP wrap_map(const boost::unordered_map<std::pair<boost::tuple<T1,T2,T3>,T4>,T5> v,
+    SEXP wrap_map(const std::unordered_map<std::pair<std::tuple<T1,T2,T3>,T4>,T5> v,
 		  std::string key, std::string name1, std::string name2);
 
 
 } // namespace Rcpp
 
-#include <Rcpp.h>
+#include <RcppArmadillo.h>
 #include <R.h>
 #include <Rmath.h>
 #include <Rdefines.h>
@@ -152,16 +176,13 @@ namespace Rcpp {
 #include <siena/ssim.h>
 #include <RngStream.h>
 #include <rcpp_table.h>
+#include <gsm.h>
 
 #include <string>
 #include <algorithm>
-//#include <map>
 #include <functional>
 #include <set>
 
-#include <boost/bind/bind.hpp>
-#include <boost/functional.hpp>
-
 namespace ssim {
 
 using std::string;
@@ -190,15 +211,22 @@ using std::greater;
 */
 class cMessage : public ssim::Event {
 public:
- cMessage(const short k = -1, const string n = "") : kind(k), name(n), sendingTime(-1.0), timestamp(0) { }
+  cMessage(const short k = -1, const string n = "", const ProcessId process_id = -1,
+	   const ProcessId sender_process_id = -1, const short priority = 0) :
+    ssim::Event(), kind(k), name(n), sendingTime(-1.0), timestamp(0), process_id(process_id),
+    sender_process_id(sender_process_id) {
+    this->priority = priority;
+  }
   // currently no setters (keep it lightweight?)
   short getKind() { return kind; }
   string getName() { return name; }
   Time getTimestamp() { return timestamp; }
   Time getSendingTime() {return sendingTime; }
+  short getSchedulePriority() {return priority; }
   short kind;
   string name;
   Time sendingTime, timestamp;
+  ProcessId process_id, sender_process_id;
   string str() const {
     std::ostringstream stringStream;
     stringStream << "kind=";
@@ -208,35 +236,196 @@ public:
     string str = stringStream.str();
     return str;
   }
-  // this does NOT include schedulePriority
 };
 
-inline bool cMessagePred(const ssim::Event* e, boost::function<bool(const cMessage * msg)> pred) {
-    const cMessage * msg = dynamic_cast<const cMessage *>(e);
-    return (msg != 0 && pred(msg));
+/**
+   @brief Cancel is a function to cancel messages who satisfy a predicate.
+
+   This works across all processes.
+*/
+  inline void
+  Cancel(std::function<bool(const cMessage * msg)> pred) {
+    return Sim::ignore_event([pred](const Event * e) {
+			       const cMessage * msg = dynamic_cast<const cMessage *>(e);
+			       return (msg != 0 && pred(msg));
+			     });
   }
 
- inline bool cMessageNamePred(const ssim::Event* e, const string s) {
-    const cMessage * msg = dynamic_cast<const cMessage *>(e);
-    return (msg != 0 && msg->name == s);
+/**
+   @brief RemoveKind is a function to remove messages with the given kind from the queue
+
+   This works across all processes.
+*/
+  inline void RemoveKind(short kind) {
+    Cancel([kind](const cMessage * msg) { return msg->kind == kind; });
   }
 
- inline bool cMessageKindPred(const ssim::Event* e, const short k) {
-    const cMessage * msg = dynamic_cast<const cMessage *>(e);
-    return (msg != 0 && msg->kind == k);
+/**
+   @brief CancelKind is a function to remove messages with the given kind from the queue
+
+   This works across all processes.
+*/
+  inline void CancelKind(short kind) {
+    Cancel([kind](const cMessage * msg) { return msg->kind == kind; });
+  }
+
+ /**
+    @brief RemoveName is a function to remove messages with the given name from the queue
+
+   This works across all processes.
+ */
+  inline void RemoveName(string name) {
+    Cancel([name](const cMessage * msg) { return msg->name == name; });
+  }
+ /**
+    @brief CancelName is a function to remove messages with the given name from the queue
+
+   This works across all processes.
+ */
+  inline void CancelName(string name) {
+    Cancel([name](const cMessage * msg) { return msg->name == name; });
+  }
+
+  /**
+     @brief CancelEvents is a function to remove messages from the queue
+
+     This works across all processes.
+  */
+inline void CancelEvents() {
+    Cancel([](const cMessage * msg) { return true; });
   }
 
 
 /**
    @brief cProcess class for OMNET++ API compatibility.
-   This provides a default for Process::process_event() that calls
-   cProcess::handleMessage(). This class also provides scheduleAt()
-   methods for insert cMessages into the process event queue.
+
+   This provides implementations for the ProcessWithPId including:
+
+   <ol>
+   <li> process_event(const Event*) that calls handleMessage(const cMessage*)
+   <li> initialize() that calls init().
+   </ol>
+
+   The user can also implement a stop() method which is called at the
+   end of a process. The scheduleAt() and send() methods insert
+   cMessages into the current and other process event queues,
+   respectively. Messages for this process can be cancelled using
+   cancel(), cancel_kind(), cancel_name() and cancel_events(). The
+   previous event time for this process can be found using previous().
+
  */
-class cProcess : public ssim::Process {
+class cProcess : public ssim::ProcessWithPId {
 public:
+  Time startTime, previousEventTime;
   cProcess(Time startTime = Time(0.0)) : startTime(startTime), previousEventTime(startTime) { }
+  /**
+      @brief Abstract method to handle each message
+   */
   virtual void handleMessage(const cMessage * msg) = 0;
+  /**
+      @brief Abstract method to initialise a given process.
+   */
+  virtual void init() = 0;
+  // see also `void stop()`
+  /**
+      @brief schedules at time t a message msg to the current process.
+      Adds the sendingTime, process_id and sender_process_id to the message.
+   */
+  virtual void scheduleAt(Time t, cMessage * msg, short priority=0) { // virtual or not?
+    msg->timestamp = t;
+    msg->sendingTime = Sim::clock();
+    msg->process_id = msg->sender_process_id = pid();
+    msg->priority = priority;
+    Sim::self_signal_event(msg, t - Sim::clock());
+  }
+  /**
+      @brief schedules at time t a message msg with a specific name to the current process.
+   */
+  virtual void scheduleAt(Time t, string name, short priority=0) {
+    scheduleAt(t, new cMessage(-1,name), priority);
+  }
+  /**
+      @brief schedules at time t a message msg with a specific kind to the current process.
+   */
+  virtual void scheduleAt(Time t, short k, short priority=0) {
+    scheduleAt(t, new cMessage(k,""), priority);
+  }
+  /**
+      @brief send to a given process at time t a message msg.
+      Adds the sendingTime, process_id and sender_process_id to the message.
+   */
+  virtual void send(ProcessId process_id, Time t, cMessage * msg, short priority=0) { // virtual or not?
+    msg->timestamp = t;
+    msg->process_id = process_id;
+    msg->sender_process_id = pid();
+    msg->sendingTime = Sim::clock();
+    msg->priority = priority;
+    Sim::signal_event(process_id, msg, t - Sim::clock());
+  }
+  /**
+      @brief sends to a given process at time t a message msg with a specific name.
+   */
+  virtual void send(ProcessId process_id, Time t, string name, short priority=0) {
+    send(process_id, t, new cMessage(-1,name), priority);
+  }
+  /**
+      @brief sends to a given process at time t a message msg with a specific kind.
+   */
+  virtual void send(ProcessId process_id, Time t, short kind, short priority=0) {
+    send(process_id, t, new cMessage(kind,""), priority);
+  }
+  /**
+      @brief Given a predicate, remove the messages for this process.
+   */
+  void cancel(std::function<bool(const cMessage * msg)> pred) {
+    Cancel([pred,this](const cMessage * msg) {
+	     return pred(msg) &&
+	       msg->process_id == this->pid();
+	   });
+  }
+  /**
+      @brief Given a kind, remove the messages for this process.
+   */
+  void cancel_kind(short kind) {
+    cancel([kind,this](const cMessage * msg) {
+	     return msg->kind == kind &&
+	       msg->process_id == this->pid();
+	   });
+  }
+  /**
+      @brief Given a name, remove the messages for this process.
+   */
+  void cancel_name(string name) {
+    cancel([name,this](const cMessage * msg) {
+	     return msg->name == name &&
+	       msg->process_id == this->pid();
+	   });
+  }
+  /**
+      @brief Remove all of the messages for this process.
+   */
+  void cancel_events() {
+    cancel([this](const cMessage * msg) {
+	     return msg->process_id == this->pid();
+	   });
+  }
+  /**
+      @brief Returns the value of the previous event time for this process.
+   */
+  double previous() {
+    return previousEventTime;
+  }
+  /**
+      @brief Implements the method required by ssim::ProcessWithPId.
+      Calls init() and sets the previousEventTime to the startTime.
+   */
+  void initialize() { init(); previousEventTime = startTime; }
+  /**
+      @brief Implements the method required by ssim::ProcessWithPId
+      This requires that the event* can be cast to a cMessage*.
+      This calls the handleMessage(const cMessage*) method and updates
+      previousEventTime.
+   */
   virtual void process_event(const ssim::Event * e) { // virtual or not?
     const cMessage * msg;
     if ((msg = dynamic_cast<const cMessage *>(e)) != 0) {
@@ -247,37 +436,8 @@ public:
       REprintf("cProcess is only written to receive cMessage events\n");
     }
   }
-  virtual void scheduleAt(Time t, cMessage * msg) { // virtual or not?
-    msg->timestamp = t;
-    msg->sendingTime = Sim::clock();
-    Sim::self_signal_event(msg, t - Sim::clock());
-  }
-  virtual void scheduleAt(Time t, string s) {
-    scheduleAt(t, new cMessage(-1,s));
-  }
-  virtual void scheduleAt(Time t, short k) {
-    scheduleAt(t, new cMessage(k,""));
-  }
-  virtual void init() = 0;
-  void initialize() { init(); previousEventTime = startTime; }
-  Time startTime, previousEventTime;
 };
 
-/**
-   @brief RemoveKind is a function to remove messages with the given kind from the queue (NB: void)
-*/
- inline void RemoveKind(short kind) {
-   Sim::ignore_event(boost::bind(cMessageKindPred,boost::placeholders::_1,kind));
- }
-
- /**
-    @brief RemoveName is a function to remove messages with the given name from the queue (NB: void)
- */
- inline void RemoveName(string name) {
-   Sim::ignore_event(boost::bind(cMessageNamePred,boost::placeholders::_1,name));
- }
-
-
 /**
    @brief simtime_t typedef for OMNET++ API compatibility
 */
@@ -349,16 +509,22 @@ public:
       }
     }
   }
- /* Rpexp(vector<double> hin, vector<double> tin) : h(hin), t(tin) { */
- /*    n = h.size(); */
- /*    H.resize(n); */
- /*    H[0] = 0.0; */
- /*    if (n>1) { */
- /*      for(i=1;i<n;i++) { */
- /* 	H[i] = H[i-1]+(t[i]-t[i-1])*h[i-1]; */
- /*      } */
- /*    } */
- /*  } */
+  Rpexp(SEXP hin_, SEXP tin_) {
+    Rcpp::NumericVector hin = Rcpp::as<Rcpp::NumericVector>(hin_);
+    Rcpp::NumericVector tin = Rcpp::as<Rcpp::NumericVector>(tin_);
+    int i;
+    n = hin.size();
+    H.resize(n);
+    t.resize(n);
+    h.resize(n);
+    H[0]=0.0; h[0]=hin[0]; t[0]=tin[0];
+    if (n>1) {
+      for(i=1;i<n;i++) {
+	h[i]=hin[i]; t[i]=tin[i];
+	H[i] = H[i-1]+(t[i]-t[i-1])*h[i-1];
+      }
+    }
+  }
   double rand(double u, double from = 0.0) {
     double v = 0.0, H0 = 0.0, tstar = 0.0;
     int i = 0, i0 = 0;
@@ -371,7 +537,6 @@ public:
     tstar = t[i]+(v-H[i])/h[i];
     return tstar;
   }
-
  private:
   vector<double> H, h, t;
   int n;
@@ -460,10 +625,31 @@ extern "C" { // functions that will be called from R
   */
 inline double discountedInterval(double start, double end, double discountRate) {
   if (discountRate == 0.0) return end - start;
-  //else if (start == 0.0) return (1.0 - (1.0+discountRate)^(-end)) / log(1.0+discountRate);
   else return (pow(1.0+discountRate,-start) - pow(1.0+discountRate,-end)) / log(1.0+discountRate);
 }
 
+  /**
+      @brief Simple function to calculate the integral between the start and end times
+      for y*(1+kappa)^(-u), where kappa is the discountRate (e.g. 0.03)
+  */
+  inline double discountedInterval(double y, double start, double end, double discountRate) {
+    if (discountRate == 0.0) return y*(end - start);
+  else return y*(pow(1.0+discountRate,-start) - pow(1.0+discountRate,-end)) / log(1.0+discountRate);
+}
+
+  /**
+     @brief Simple function to calculate 1/(1+discountRate)^(time)
+  */
+  inline double discountedPoint(double time, double discountRate) {
+    return discountRate <= 0.0 ? 1.0 : pow(1.0+discountRate,-time);
+  }
+  /**
+     @brief Simple function to calculate y/(1+discountRate)^(time)
+  */
+  inline double discountedPoint(double y, double time, double discountRate) {
+    return discountRate <= 0.0 ? y : y*pow(1.0+discountRate,-time);
+  }
+
 
 /**
     @brief Function to transpose a vector of vectors.
@@ -492,14 +678,15 @@ inline double discountedInterval(double start, double end, double discountRate)
  typedef std::set<Time, std::greater<Time> > Partition; // NB: greater<Time> cf. lesser<Time>
  typedef typename Partition::iterator Iterator;
  typedef std::pair<State,Time> Pair;
- typedef boost::unordered_map<pair<State,Time>, int > PrevMap;
- typedef boost::unordered_map<pair<State,Time>, Utility > UtilityMap;
- typedef boost::unordered_map<pair<State,Time>, Time > PtMap;
- typedef boost::unordered_map<boost::tuple<State,Event,Time>, int > EventsMap;
+ typedef std::unordered_map<pair<State,Time>, int > PrevMap;
+ typedef std::unordered_map<pair<State,Time>, Utility > UtilityMap;
+ typedef std::unordered_map<pair<State,Time>, Time > PtMap;
+ typedef std::unordered_map<std::tuple<State,Event,Time>, int > EventsMap;
  typedef vector<Utility> IndividualUtilities;
    EventReport(Utility discountRate = 0.0, bool outputUtilities = true, int size = 1, Time startReportAge = Time(0), bool indiv = false) :
      discountRate(discountRate), outputUtilities(outputUtilities), startReportAge(startReportAge), id(0), indiv(indiv) {
    _vector.resize(size);
+   setPartition(startReportAge);
  }
  void resize(int size) {
    _vector.resize(size);
@@ -507,7 +694,8 @@ inline double discountedInterval(double start, double end, double discountRate)
  void setPartition(const vector<Time> v) {
    copy(v.begin(), v.end(), inserter(_partition, _partition.begin()));
   }
- void setPartition(const Time start, const Time finish, const Time delta,
+ void setPartition(const Time start = 0.0, const Time finish = 100.0,
+		   const Time delta = 1.0,
 		   const Time maxTime = Time(1.0e100)) {
    _partition.clear();
    for (Time t=start; t<=finish; t+=delta) _partition.insert(t);
@@ -563,7 +751,7 @@ inline double discountedInterval(double start, double end, double discountRate)
    lo = _partition.lower_bound(lhs);
    hi = _partition.lower_bound(rhs);
    last = _partition.begin(); // because it is ordered by greater<Time>
-   ++_events[boost::make_tuple(state,event,*hi)];
+   ++_events[std::make_tuple(state,event,*hi)];
    bool iterating;
    for(it = lo, iterating = true; iterating; iterating = (it != hi), --it) { // decrement for greater<Time>
       if (lhs<=(*it) && (*it)<rhs) // cadlag
@@ -640,11 +828,11 @@ inline double discountedInterval(double start, double end, double discountRate)
     typedef std::set<Time, std::greater<Time> > Partition; // NB: greater<Time> cf. lesser<Time>
     typedef typename Partition::iterator Iterator;
     typedef std::pair<State,Time> Pair;
-    typedef boost::unordered_map<pair<State,Time>, int > PrevMap;
-    typedef boost::unordered_map<pair<State,Time>, Utility > UtilityMap;
-    typedef boost::unordered_map<pair<State,Time>, Time > PtMap;
-    typedef boost::unordered_map<boost::tuple<State,Event,Time>, int > EventMap;
-    typedef boost::unordered_map<pair<State,Time>, Cost > CostMap;
+    typedef std::unordered_map<pair<State,Time>, int > PrevMap;
+    typedef std::unordered_map<pair<State,Time>, Utility > UtilityMap;
+    typedef std::unordered_map<pair<State,Time>, Time > PtMap;
+    typedef std::unordered_map<std::tuple<State,Event,Time>, int > EventMap;
+    typedef std::unordered_map<pair<State,Time>, Cost > CostMap;
     typedef std::vector<Cost> IndividualCosts;
     typedef std::vector<Utility> IndividualUtilities;
     SummaryReport(int n = 1, bool indivp = true, Utility discountRate = 0.0) : n(n), indivp(indivp) {
@@ -702,7 +890,7 @@ inline double discountedInterval(double start, double end, double discountRate)
       lo = _partition.lower_bound(lhs);
       hi = _partition.lower_bound(rhs);
       last = _partition.begin(); // because it is ordered by greater<Time>
-      ++_events[boost::make_tuple(state,event,*hi)];
+      ++_events[std::make_tuple(state,event,*hi)];
       bool iterating;
       for(it = lo, iterating = true; iterating; iterating = (it != hi), --it) { // decrement for greater<Time>
 	if (lhs<=(*it) && (*it)<rhs) // cadlag
@@ -827,7 +1015,7 @@ inline double discountedInterval(double start, double end, double discountRate)
  typedef std::set<Time, std::greater<Time> > Partition;
  typedef std::pair<State,Time> Pair;
  typedef CostReport<State,Time,Cost> This;
- typedef boost::unordered_map<pair<State,Time>, Cost > Table;
+ typedef std::unordered_map<pair<State,Time>, Cost > Table;
  typedef std::vector<Cost> IndividualCosts;
    CostReport(Cost discountRate = 0, int size = 1, Time startReportAge = Time(0), bool indiv = false) : discountRate(discountRate), startReportAge(startReportAge), id(0), indiv(indiv) {
    _vector.resize(size);
@@ -980,67 +1168,67 @@ namespace Rcpp {
   }
 
   template <class T1, class T2>
-    SEXP wrap(const vector<boost::tuple<T1,T2> > v) {
+    SEXP wrap(const vector<std::tuple<T1,T2> > v) {
     int i, n=v.size();
     vector<T1> v1(n);
     vector<T2> v2(n);
-    typename vector<boost::tuple<T1,T2> >::const_iterator it;
+    typename vector<std::tuple<T1,T2> >::const_iterator it;
     for (it=v.begin(), i=0; it<v.end(); ++it, ++i) {
-      v1[i] = boost::get<0>(*it);
-      v2[i] = boost::get<1>(*it);
+      v1[i] = std::get<0>(*it);
+      v2[i] = std::get<1>(*it);
     }
     return List::create(_("Var1")=wrap(v1),_("Var2")=wrap(v2));
   }
 
   template <class T1, class T2, class T3>
-    SEXP wrap(const vector<boost::tuple<T1,T2,T3> > v) {
+    SEXP wrap(const vector<std::tuple<T1,T2,T3> > v) {
     int i, n=v.size();
     vector<T1> v1(n);
     vector<T2> v2(n);
     vector<T3> v3(n);
-    typename vector<boost::tuple<T1,T2,T3> >::const_iterator it;
+    typename vector<std::tuple<T1,T2,T3> >::const_iterator it;
     for (it=v.begin(), i=0; it<v.end(); ++it, ++i) {
-      v1[i] = boost::get<0>(*it);
-      v2[i] = boost::get<1>(*it);
-      v3[i] = boost::get<2>(*it);
+      v1[i] = std::get<0>(*it);
+      v2[i] = std::get<1>(*it);
+      v3[i] = std::get<2>(*it);
     }
     return List::create(_("Var1")=wrap(v1),_("Var2")=wrap(v2),_("Var3")=wrap(v3));
   }
 
 
   template <class T1, class T2, class T3, class T4>
-    SEXP wrap(const vector<boost::tuple<T1,T2,T3,T4> > v) {
+    SEXP wrap(const vector<std::tuple<T1,T2,T3,T4> > v) {
     int i, n=v.size();
     vector<T1> v1(n);
     vector<T2> v2(n);
     vector<T3> v3(n);
     vector<T4> v4(n);
-    typename vector<boost::tuple<T1,T2,T3,T4> >::const_iterator it;
+    typename vector<std::tuple<T1,T2,T3,T4> >::const_iterator it;
     for (it=v.begin(), i=0; it<v.end(); ++it, ++i) {
-      v1[i] = boost::get<0>(*it);
-      v2[i] = boost::get<1>(*it);
-      v3[i] = boost::get<2>(*it);
-      v4[i] = boost::get<3>(*it);
+      v1[i] = std::get<0>(*it);
+      v2[i] = std::get<1>(*it);
+      v3[i] = std::get<2>(*it);
+      v4[i] = std::get<3>(*it);
     }
     return List::create(_("Var1")=wrap(v1),_("Var2")=wrap(v2),
 			_("Var3")=wrap(v3),_("Var4")=wrap(v4));
   }
 
   template <class T1, class T2, class T3, class T4, class T5>
-    SEXP wrap(const vector<boost::tuple<T1,T2,T3,T4,T5> > v) {
+    SEXP wrap(const vector<std::tuple<T1,T2,T3,T4,T5> > v) {
     int i, n=v.size();
     vector<T1> v1(n);
     vector<T2> v2(n);
     vector<T3> v3(n);
     vector<T4> v4(n);
     vector<T5> v5(n);
-    typename vector<boost::tuple<T1,T2,T3,T4,T5> >::const_iterator it;
+    typename vector<std::tuple<T1,T2,T3,T4,T5> >::const_iterator it;
     for (it=v.begin(), i=0; it<v.end(); ++it, ++i) {
-      v1[i] = boost::get<0>(*it);
-      v2[i] = boost::get<1>(*it);
-      v3[i] = boost::get<2>(*it);
-      v4[i] = boost::get<3>(*it);
-      v5[i] = boost::get<4>(*it);
+      v1[i] = std::get<0>(*it);
+      v2[i] = std::get<1>(*it);
+      v3[i] = std::get<2>(*it);
+      v4[i] = std::get<3>(*it);
+      v5[i] = std::get<4>(*it);
     }
     return List::create(_("Var1")=wrap(v1),_("Var2")=wrap(v2),
 			_("Var3")=wrap(v3),_("Var4")=wrap(v4),
@@ -1048,7 +1236,7 @@ namespace Rcpp {
   }
 
   template <class T1, class T2, class T3, class T4, class T5, class T6>
-    SEXP wrap(const vector<boost::tuple<T1,T2,T3,T4,T5,T6> > v) {
+    SEXP wrap(const vector<std::tuple<T1,T2,T3,T4,T5,T6> > v) {
     int i, n=v.size();
     vector<T1> v1(n);
     vector<T2> v2(n);
@@ -1056,14 +1244,14 @@ namespace Rcpp {
     vector<T4> v4(n);
     vector<T5> v5(n);
     vector<T6> v6(n);
-    typename vector<boost::tuple<T1,T2,T3,T4,T5,T6> >::const_iterator it;
+    typename vector<std::tuple<T1,T2,T3,T4,T5,T6> >::const_iterator it;
     for (it=v.begin(), i=0; it<v.end(); ++it, ++i) {
-      v1[i] = boost::get<0>(*it);
-      v2[i] = boost::get<1>(*it);
-      v3[i] = boost::get<2>(*it);
-      v4[i] = boost::get<3>(*it);
-      v5[i] = boost::get<4>(*it);
-      v6[i] = boost::get<5>(*it);
+      v1[i] = std::get<0>(*it);
+      v2[i] = std::get<1>(*it);
+      v3[i] = std::get<2>(*it);
+      v4[i] = std::get<3>(*it);
+      v5[i] = std::get<4>(*it);
+      v6[i] = std::get<5>(*it);
     }
     return List::create(_("Var1")=wrap(v1),_("Var2")=wrap(v2),
 			_("Var3")=wrap(v3),_("Var4")=wrap(v4),
@@ -1071,7 +1259,7 @@ namespace Rcpp {
   }
 
   template <class T1, class T2, class T3, class T4, class T5, class T6, class T7>
-    SEXP wrap(const vector<boost::tuple<T1,T2,T3,T4,T5,T6,T7> > v) {
+    SEXP wrap(const vector<std::tuple<T1,T2,T3,T4,T5,T6,T7> > v) {
     int i, n=v.size();
     vector<T1> v1(n);
     vector<T2> v2(n);
@@ -1080,15 +1268,15 @@ namespace Rcpp {
     vector<T5> v5(n);
     vector<T6> v6(n);
     vector<T7> v7(n);
-    typename vector<boost::tuple<T1,T2,T3,T4,T5,T6,T7> >::const_iterator it;
+    typename vector<std::tuple<T1,T2,T3,T4,T5,T6,T7> >::const_iterator it;
     for (it=v.begin(), i=0; it<v.end(); ++it, ++i) {
-      v1[i] = boost::get<0>(*it);
-      v2[i] = boost::get<1>(*it);
-      v3[i] = boost::get<2>(*it);
-      v4[i] = boost::get<3>(*it);
-      v5[i] = boost::get<4>(*it);
-      v6[i] = boost::get<5>(*it);
-      v7[i] = boost::get<6>(*it);
+      v1[i] = std::get<0>(*it);
+      v2[i] = std::get<1>(*it);
+      v3[i] = std::get<2>(*it);
+      v4[i] = std::get<3>(*it);
+      v5[i] = std::get<4>(*it);
+      v6[i] = std::get<5>(*it);
+      v7[i] = std::get<6>(*it);
     }
     return List::create(_("Var1")=wrap(v1),_("Var2")=wrap(v2),
 			_("Var3")=wrap(v3),_("Var4")=wrap(v4),
@@ -1097,7 +1285,7 @@ namespace Rcpp {
   }
 
   template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8>
-    SEXP wrap(const vector<boost::tuple<T1,T2,T3,T4,T5,T6,T7,T8> > v) {
+    SEXP wrap(const vector<std::tuple<T1,T2,T3,T4,T5,T6,T7,T8> > v) {
     int i, n=v.size();
     vector<T1> v1(n);
     vector<T2> v2(n);
@@ -1107,16 +1295,16 @@ namespace Rcpp {
     vector<T6> v6(n);
     vector<T7> v7(n);
     vector<T8> v8(n);
-    typename vector<boost::tuple<T1,T2,T3,T4,T5,T6,T7,T8> >::const_iterator it;
+    typename vector<std::tuple<T1,T2,T3,T4,T5,T6,T7,T8> >::const_iterator it;
     for (it=v.begin(), i=0; it<v.end(); ++it, ++i) {
-      v1[i] = boost::get<0>(*it);
-      v2[i] = boost::get<1>(*it);
-      v3[i] = boost::get<2>(*it);
-      v4[i] = boost::get<3>(*it);
-      v5[i] = boost::get<4>(*it);
-      v6[i] = boost::get<5>(*it);
-      v7[i] = boost::get<6>(*it);
-      v8[i] = boost::get<7>(*it);
+      v1[i] = std::get<0>(*it);
+      v2[i] = std::get<1>(*it);
+      v3[i] = std::get<2>(*it);
+      v4[i] = std::get<3>(*it);
+      v5[i] = std::get<4>(*it);
+      v6[i] = std::get<5>(*it);
+      v7[i] = std::get<6>(*it);
+      v8[i] = std::get<7>(*it);
     }
     return List::create(_("Var1")=wrap(v1),_("Var2")=wrap(v2),
 			_("Var3")=wrap(v3),_("Var4")=wrap(v4),
@@ -1125,7 +1313,7 @@ namespace Rcpp {
   }
 
   template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9>
-    SEXP wrap(const vector<boost::tuple<T1,T2,T3,T4,T5,T6,T7,T8,T9> > v) {
+    SEXP wrap(const vector<std::tuple<T1,T2,T3,T4,T5,T6,T7,T8,T9> > v) {
     int i, n=v.size();
     vector<T1> v1(n);
     vector<T2> v2(n);
@@ -1136,17 +1324,17 @@ namespace Rcpp {
     vector<T7> v7(n);
     vector<T8> v8(n);
     vector<T9> v9(n);
-    typename vector<boost::tuple<T1,T2,T3,T4,T5,T6,T7,T8,T9> >::const_iterator it;
+    typename vector<std::tuple<T1,T2,T3,T4,T5,T6,T7,T8,T9> >::const_iterator it;
     for (it=v.begin(), i=0; it<v.end(); ++it, ++i) {
-      v1[i] = boost::get<0>(*it);
-      v2[i] = boost::get<1>(*it);
-      v3[i] = boost::get<2>(*it);
-      v4[i] = boost::get<3>(*it);
-      v5[i] = boost::get<4>(*it);
-      v6[i] = boost::get<5>(*it);
-      v7[i] = boost::get<6>(*it);
-      v8[i] = boost::get<7>(*it);
-      v9[i] = boost::get<8>(*it);
+      v1[i] = std::get<0>(*it);
+      v2[i] = std::get<1>(*it);
+      v3[i] = std::get<2>(*it);
+      v4[i] = std::get<3>(*it);
+      v5[i] = std::get<4>(*it);
+      v6[i] = std::get<5>(*it);
+      v7[i] = std::get<6>(*it);
+      v8[i] = std::get<7>(*it);
+      v9[i] = std::get<8>(*it);
     }
     return List::create(_("Var1")=wrap(v1),_("Var2")=wrap(v2),
 			_("Var3")=wrap(v3),_("Var4")=wrap(v4),
@@ -1156,7 +1344,7 @@ namespace Rcpp {
   }
 
   template <class T1, class T2, class T3, class T4, class T5, class T6, class T7, class T8, class T9, class T10>
-    SEXP wrap(const vector<boost::tuple<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10> > v) {
+    SEXP wrap(const vector<std::tuple<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10> > v) {
     int i, n=v.size();
     vector<T1> v1(n);
     vector<T2> v2(n);
@@ -1168,18 +1356,18 @@ namespace Rcpp {
     vector<T8> v8(n);
     vector<T9> v9(n);
     vector<T10> v10(n);
-    typename vector<boost::tuple<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10> >::const_iterator it;
+    typename vector<std::tuple<T1,T2,T3,T4,T5,T6,T7,T8,T9,T10> >::const_iterator it;
     for (it=v.begin(), i=0; it<v.end(); ++it, ++i) {
-      v1[i] = boost::get<0>(*it);
-      v2[i] = boost::get<1>(*it);
-      v3[i] = boost::get<2>(*it);
-      v4[i] = boost::get<3>(*it);
-      v5[i] = boost::get<4>(*it);
-      v6[i] = boost::get<5>(*it);
-      v7[i] = boost::get<6>(*it);
-      v8[i] = boost::get<7>(*it);
-      v9[i] = boost::get<8>(*it);
-      v10[i] = boost::get<9>(*it);
+      v1[i] = std::get<0>(*it);
+      v2[i] = std::get<1>(*it);
+      v3[i] = std::get<2>(*it);
+      v4[i] = std::get<3>(*it);
+      v5[i] = std::get<4>(*it);
+      v6[i] = std::get<5>(*it);
+      v7[i] = std::get<6>(*it);
+      v8[i] = std::get<7>(*it);
+      v9[i] = std::get<8>(*it);
+      v10[i] = std::get<9>(*it);
     }
     return List::create(_("Var1")=wrap(v1),_("Var2")=wrap(v2),
 			_("Var3")=wrap(v3),_("Var4")=wrap(v4),
@@ -1207,12 +1395,12 @@ namespace Rcpp {
   //
   // map<pair<Tstate,T>, int > _prev;
   // map<pair<Tstate,T>, T > _pt;
-  // map<boost::tuple<Tstate,Tevent,T>, int > _events;
+  // map<std::tuple<Tstate,Tevent,T>, int > _events;
 
   template <class T1a, class T1b, class T1c, class T2>
-    SEXP wrap_map(const std::map<boost::tuple<T1a,T1b,T1c>,T2> v,
+    SEXP wrap_map(const std::map<std::tuple<T1a,T1b,T1c>,T2> v,
 		  std::string key, std::string name1, std::string name2, std::string name3) {
-    typedef boost::tuple<T1a,T1b,T1c> Tuple;
+    typedef std::tuple<T1a,T1b,T1c> Tuple;
     int i;
     int n = v.size();
     vector<T1a> xa(n);
@@ -1252,29 +1440,29 @@ namespace Rcpp {
 
 
   template <class T1, class T2>
-    SEXP wrap_map(const boost::unordered_map<T1,T2> ov) {
+    SEXP wrap_map(const std::unordered_map<T1,T2> ov) {
     std::map<T1,T2> v(ov.begin(), ov.end());
     return wrap_map<T1,T2>(v);
   }
 
   template <class T1a, class T1b, class T1c, class T2>
-    SEXP wrap_map(const boost::unordered_map<boost::tuple<T1a,T1b,T1c>,T2> ov,
+    SEXP wrap_map(const std::unordered_map<std::tuple<T1a,T1b,T1c>,T2> ov,
 		  std::string key, std::string name1, std::string name2, std::string name3) {
-    std::map<boost::tuple<T1a,T1b,T1c>,T2> v(ov.begin(), ov.end());
+    std::map<std::tuple<T1a,T1b,T1c>,T2> v(ov.begin(), ov.end());
     return wrap_map<T1a, T1b, T1c, T2>(v, key, name1, name2, name3);
   }
 
   template <class T1a, class T1b, class T2>
-    SEXP wrap_map(const boost::unordered_map<std::pair<T1a,T1b>,T2> ov,
+    SEXP wrap_map(const std::unordered_map<std::pair<T1a,T1b>,T2> ov,
 		  std::string key, std::string name1, std::string name2) {
     std::map<std::pair<T1a,T1b>,T2> v(ov.begin(), ov.end());
     return wrap_map<T1a, T1b, T2>(v, key, name1, name2);
    }
 
   template <class T1, class T2, class T3, class T4, class T5>
-  SEXP wrap_map(const std::map<std::pair<boost::tuple<T1,T2,T3>,T4>,T5> v,
+  SEXP wrap_map(const std::map<std::pair<std::tuple<T1,T2,T3>,T4>,T5> v,
 		std::string name1, std::string name2) {
-    typedef boost::tuple<T1,T2,T3> Tuple;
+    typedef std::tuple<T1,T2,T3> Tuple;
     typedef std::pair<Tuple,T4> Pair;
     int i;
     int n = v.size();
@@ -1286,9 +1474,9 @@ namespace Rcpp {
     typename std::map<Pair,T5>::const_iterator it;
     for (it=v.begin(), i=0; it != v.end(); ++it, ++i) {
       Tuple tuple = (*it).first.first;
-      x1[i] = boost::get<0>(tuple);
-      x2[i] = boost::get<1>(tuple);
-      x3[i] = boost::get<2>(tuple);
+      x1[i] = std::get<0>(tuple);
+      x2[i] = std::get<1>(tuple);
+      x3[i] = std::get<2>(tuple);
       x4[i] = (*it).first.second;
       y[i] = (*it).second;
     }
@@ -1301,9 +1489,9 @@ namespace Rcpp {
    }
 
   template <class T1, class T2, class T3, class T4, class T5>
-  SEXP wrap_map(const boost::unordered_map<std::pair<boost::tuple<T1,T2,T3>,T4>,T5> ov,
+  SEXP wrap_map(const std::unordered_map<std::pair<std::tuple<T1,T2,T3>,T4>,T5> ov,
 		std::string name1, std::string name2) {
-    std::map<std::pair<boost::tuple<T1,T2,T3>,T4>,T5> v(ov.begin(), ov.end());
+    std::map<std::pair<std::tuple<T1,T2,T3>,T4>,T5> v(ov.begin(), ov.end());
     return wrap_map<T1,T2,T3,T4,T5>(v, name1, name2);
    }
 
@@ -1314,23 +1502,33 @@ namespace Rcpp {
 
 namespace R {
   /**
-     @brief rnorm function constrained to be positive. This uses brute-force re-sampling rather
-     than conditioning on the distribution function.
+     @brief rnorm function constrained to be positive. This uses
+     brute-force re-sampling rather than conditioning on the
+     distribution function.
   */
   double rnormPos(double mean, double sd);
 
   /**
-     @brief rllogis function for a random covariate from a log-logistic distribution with shape and scale.
-     S(t) = 1/(1+(t/scale)^shape).
+     @brief rllogis function for a random covariate from a
+     log-logistic distribution with shape and scale.  S(t) =
+     1/(1+(t/scale)^shape).
   */
   double rllogis(double shape, double scale);
   /**
-     @brief rllogis_trunc function for a random covariate from a log-logistic distribution with shape and scale
-     with minimum time left.
-     S(t|t>x)=S(t)/S(x) where S(t)=1/(1+(t/scale)^shape).
+     @brief rllogis_trunc function for a random covariate from a
+     log-logistic distribution with shape and scale with minimum time
+     left.  S(t|t>x)=S(t)/S(x) where S(t)=1/(1+(t/scale)^shape).
   */
   double rllogis_trunc(double shape, double scale, double left);
+  /**
+     @brief Random draw from a Gompertz distribution.
 
+     Importantly, the parameterisation is as per flexsurv, where
+     hazard(t)=rate*exp(shape*t).  As a reminder, if shape<0 then
+     there is a non-zero probability of cure (that is, no event),
+     which is represented by R_PosInf.
+  */
+  double rgompertz(double shape = 1.0, double rate = 1.0);
 }
 
 #endif
